import 'package:flutter/material.dart';
import 'package:json_annotation/json_annotation.dart';
import '../json/common.dart';
import '../json/common_platform.dart';
import '../barcode_scanning_data.dart';
import '../sdk_utils.dart';

import 'camera_configuration.dart';
export 'finder.dart';

part 'barcode_scanner_configuration.g.dart';

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class BarcodeCameraConfiguration extends CameraConfiguration {
  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool? flashEnabled;

  /// Set current zoom by a linear zoom value ranging from 0f to 1.0f. Optical Zoom Level 0f represents the minimum zoom while Optical Zoom Level 1.0f represents the maximum zoom.
  /// Default value is 0f.
  double? cameraZoomFactor;

  @override
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.barcode;

  /// Configuration of the finder window.
  FinderConfiguration? finder;

  /// Configuration of the barcode scanner detection behavior eg. which barcode types to detect etc.
  BarcodeClassicScannerConfiguration scannerConfiguration;

  ///The configuration of AR selection overlay.
  SelectionOverlayScannerConfiguration? overlayConfiguration;

  BarcodeCameraConfiguration({
    required this.scannerConfiguration,
    this.flashEnabled,
    this.finder,
    this.cameraZoomFactor,
    this.overlayConfiguration,
  });

  @override
  Map<String, dynamic> toJson() => _$BarcodeCameraConfigurationToJson(this);
}

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class BarcodeClassicScannerConfiguration {
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.barcode;

  /// List of accepted Barcode Formats; any formats that are not included in this
  /// list will not be detected by the `BarcodeCameraView`. By default, the list
  /// includes all supported barcode types.
  List<BarcodeFormat> barcodeFormats;

  /// List of accepted Document Formats; any document formats that are not included
  /// in this list will be ignored by the `BarcodeCameraView`. By default, the list
  /// includes all supported document types.
  List<BarcodeDocumentFormat> acceptedDocumentFormats;

  /// The engine mode of the barcode recognizer. Defaults to NEXT_GEN.
  /// To use legacy recognizer, please specify LEGACY
  EngineMode? engineMode;

  /// Additional parameters for tweaking the detection of barcodes.
  BarcodeAdditionalParameters? additionalParameters;

  /// Param that handle whether we need to save image from camera preview or make a snapshot while snapping barcode
  /// image uri will be returned in BarcodeScanningResult.barcodeImageURI` property
  BarcodeImageGenerationType barcodeImageGenerationType =
      BarcodeImageGenerationType.NONE;

  BarcodeClassicScannerConfiguration(
      {this.barcodeFormats = const [],
      this.acceptedDocumentFormats = const [],
      this.engineMode = EngineMode.NEXT_GEN,
      this.additionalParameters,
      this.barcodeImageGenerationType = BarcodeImageGenerationType.NONE});

  Map<String, dynamic> toJson() =>
      _$BarcodeClassicScannerConfigurationToJson(this);
}

/// Barcode Selection Overlay configuration
@JsonSerializable(
  includeIfNull: false,
  createFactory: false,
)
class SelectionOverlayScannerConfiguration {
  /// Whether the barcode selection overlay is enabled or not.
  bool overlayEnabled;

  /// Whether to return results from barcodeLiveDetection callback or only proceed with manual click on barcode polygon
  bool automaticSelectionEnabled;

  /// Define the way of how to show barcode data with selection overlay.
  BarcodeOverlayTextFormat textFormat;

  /// The color of the polygon in the selection overlay.
  @JsonKey(toJson: toColorString)
  Color? polygonColor = Colors.white;

  /// The color of the text in the selection overlay.
  @JsonKey(toJson: toColorString)
  Color? textColor = Colors.black;

  /// The color of the texts background in the selection overlay.
  @JsonKey(toJson: toColorString)
  Color? textContainerColor = Colors.white;

  /// Callback that will be called when barcode on screen is clicked.
  @JsonKey(includeToJson: false)
  Function(BarcodeItem item)? onBarcodeClicked;

  SelectionOverlayScannerConfiguration({
    this.overlayEnabled = true,
    this.automaticSelectionEnabled = false,
    this.textFormat = BarcodeOverlayTextFormat.CODE_AND_TYPE,
    this.polygonColor = Colors.white,
    this.textColor = Colors.black,
    this.textContainerColor = Colors.white,
    this.onBarcodeClicked,
  });

  Map<String, dynamic> toJson() =>
      _$SelectionOverlayScannerConfigurationToJson(this);
}
