import 'dart:ui';

import 'package:json_annotation/json_annotation.dart';

import '../sdk_utils.dart';
import '../json/common_platform.dart';
import 'camera_configuration.dart';

part 'document_scanner_configuration.g.dart';

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class DocumentCameraConfiguration extends CameraConfiguration {
  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool? flashEnabled;

  /// Set current zoom by a linear zoom value ranging from 0f to 1.0f. Optical Zoom Level 0f represents the minimum zoom while Optical Zoom Level 1.0f represents the maximum zoom.
  /// Default value is 0f.
  double? cameraZoomFactor;

  @override
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.document;

  /// Configuration of the document contour appearance.
  ContourConfiguration? contourConfiguration;

  /// Configuration of the document scanner detection behavior eg. which searched documents aspect ratios etc.
  DocumentClassicScannerConfiguration scannerConfiguration;

  DocumentCameraConfiguration({
    required this.scannerConfiguration,
    this.flashEnabled,
    this.contourConfiguration,
    this.cameraZoomFactor,
  });

  @override
  Map<String, dynamic> toJson() => _$DocumentCameraConfigurationToJson(this);
}

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class DocumentClassicScannerConfiguration {
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.document;

  /// Should detect document after snap?
  /// If true it will run document detection on Original image and will create document image
  /// Default - false
  bool? detectDocumentAfterSnap;

  /// Whether the auto-snapping after successful document detection is enabled or not.
  bool autoSnapEnabled;

  bool? ignoreBadAspectRatio;

  /// The minimum score in percent (0 - 100) of the perspective distortion to accept a detected document. Default is 75.0.
  double? acceptedAngleScore;

  /// The minimum document width or height in percent (0 - 100) of the screen size to accept a detected document. Default is 80.0.
  double? acceptedSizeScore;

  /// Expected aspect ratios of the document. If the document has a different aspect ratio it will be rejected.
  List<PageAspectRatio>? requiredAspectRatios;

  /// Changes sensitivity of auto-snapping. That is: the more sensitive it is the faster it shoots.
  /// This parameter only works when `barcodeImageGenerationType = BarcodeImageGenerationType.CAPTURED_IMAGE`
  /// Sensitivity must be within (0..1) range. A value of 1.0 triggers automatic capturing immediately,
  /// a value of 0.0 delays the automatic capturing by 3 seconds.
  ///
  /// The default value is 0.66 (1 sec) so you will get snapped barcode result with saved image each not often than 1 second.
  ///
  /// @param sensitivity should be within (0..1) range.
  double? autoSnapSensitivity;

  /// [originalImageSizeLimit] the maximum size of the original image to be saved. If the original image is larger than the specified size it will be downscaled. By default not specified.
  Size? originalImageSizeLimit;

  /// [documentImageSizeLimit] the maximum size of the cropped document image to be saved. If the image is larger than the specified size it will be downscaled. If [originalImageSizeLimit] is set it also limits the size of the cropped document image. By default not specified.

  Size? documentImageSizeLimit;

  DocumentClassicScannerConfiguration({
    this.ignoreBadAspectRatio,
    this.detectDocumentAfterSnap,
    this.autoSnapEnabled = false,
    this.acceptedAngleScore,
    this.acceptedSizeScore,
    this.requiredAspectRatios,
    this.autoSnapSensitivity,
    this.originalImageSizeLimit,
    this.documentImageSizeLimit,
  });

  Map<String, dynamic> toJson() =>
      _$DocumentClassicScannerConfigurationToJson(this);
}

@JsonSerializable(includeIfNull: false, explicitToJson: true)
class PageAspectRatio {
  final double height;

  final double width;

  const PageAspectRatio({required this.width, required this.height});

  factory PageAspectRatio.fromJson(Map<String, dynamic> json) =>
      _$PageAspectRatioFromJson(json);

  Map<String, dynamic> toJson() => _$PageAspectRatioToJson(this);
}

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class ContourConfiguration {
  /// Whether to show polygon when user scans document with `autoSnapEnabled == false`. Default is false.
  bool showPolygonInManualMode;

  @JsonKey(toJson: toColorString)
  Color? autoSnapProgressStrokeColor;

  @JsonKey(toJson: toColorString)
  Color? strokeColor;

  @JsonKey(toJson: toColorString)
  Color? strokeOkColor;

  @JsonKey(toJson: toColorString)
  Color? fillColor;

  @JsonKey(toJson: toColorString)
  Color? fillOkColor;

  double? strokeWidth;

  double? autoSnapProgressStrokeWidth;
  bool? autoSnapProgressEnabled;

  double? cornerRadius;

  ContourConfiguration(
      {this.strokeColor,
      this.strokeOkColor,
      this.fillColor,
      this.fillOkColor,
      this.strokeWidth,
      this.autoSnapProgressStrokeColor,
      this.autoSnapProgressStrokeWidth,
      this.autoSnapProgressEnabled,
      this.showPolygonInManualMode = false,
      this.cornerRadius});

  Map<String, dynamic> toJson() => _$ContourConfigurationToJson(this);
}
