import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:logging/logging.dart';

import 'scanbot_sdk.dart';

class ScanbotSdkUi {
  static const MethodChannel _batchBarcodeFormatterCallbackChannel =
      MethodChannel('batch_barcode_data_mapper');
  static const MethodChannel _channel = MethodChannel('scanbot_sdk');

  static Future<String> _invoke(method, Map<String, dynamic> json) async {
    return await _channel.invokeMethod(method, json);
  }

  static bool _wasCanceled(String result) {
    return result == 'CANCELED';
  }

  /// Starts the Ready-to-Use Barcode Scanner UI as full-screen modal UI dialog.
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  static Future<BarcodeScanningResult> startBarcodeScanner(
      BarcodeScannerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();
      const defaultJsonName = "BarcodeScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var result = await _invoke('startBarcodeScanner', configuration);
      if (_wasCanceled(result)) {
        return BarcodeScanningResult.canceled();
      }
      return BarcodeScanningResult.fromJson(jsonDecode(result));
    } catch (e) {
      Logger.root.severe(e);
      return BarcodeScanningResult.error(e.toString());
    }
  }

  static Handler? _handler;

  static void _initBatchFormatterHandler(
      MethodChannel sessionCallbackChannel, BarcodeDataFormatter formatter) {
    _handler = (MethodCall call) async {
      if (call.method == 'getFormattedData') {
        var dataItem =
            await formatter(BarcodeItem.fromJson(jsonDecode(call.arguments)));
        return dataItem.toJson();
      }
    };
  }

  /// Starts the Ready-to-Use Batch Barcode Scanner UI as full-screen modal UI dialog.
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  static Future<BarcodeScanningResult> startBatchBarcodeScanner(
      BatchBarcodeScannerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();

      if (config.barcodeFormatter != null) {
        _initBatchFormatterHandler(
            _batchBarcodeFormatterCallbackChannel, config.barcodeFormatter!);
        _batchBarcodeFormatterCallbackChannel.setMethodCallHandler(_handler);
        updatedParameters['shouldUseBarcodeFormatter'] = true;
      } else {
        _handler = null;
      }

      const defaultJsonName = "DocumentScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var barcodeScanningResult = await _channel.invokeMethod(
          'startBatchBarcodeScanner', configuration);
      _handler = null;
      if (_wasCanceled(barcodeScanningResult)) {
        return BarcodeScanningResult.canceled();
      }
      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      Logger.root.severe(e);
      return BarcodeScanningResult.error(e.toString());
    }
  }

  /// Starts the Ready-to-Use Document Scanner UI as full-screen modal UI dialog.
  /// The Document Scanner provides guided, automatic document scanning.
  static Future<DocumentScanningResult> startDocumentScanner(
      DocumentScannerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();
      const defaultJsonName = "DocumentScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var docSize = configuration['documentImageSizeLimit'];
      if (docSize['width'] == 0 || docSize['height'] == 0) {
        configuration.remove('documentImageSizeLimit');
      }

      var result = await _invoke('startDocumentScanner', configuration);
      if (_wasCanceled(result)) {
        return DocumentScanningResult.canceled();
      }
      return DocumentScanningResult.fromJson(jsonDecode(result));
    } catch (e) {
      Logger.root.severe(e);
      return DocumentScanningResult.error(e.toString());
    }
  }

  /// Starts the Ready-to-Use Generic Document Recognizer as a full-screen modal UI dialog.
  /// The Generic Document Recognizer provides guided, automatic document scanning.
  static Future<GenericDocumentWrappedResults> startGenericDocumentRecognizer(
      GenericDocumentRecognizerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();
      const defaultJsonName = "GenericDocumentScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var result =
          await _invoke('startGenericDocumentRecognizer', configuration);
      if (_wasCanceled(result)) {
        return GenericDocumentWrappedResults.canceled();
      }
      var genericResult = GenericDocumentResults.fromJson(jsonDecode(result));
      return GenericDocumentWrappedResults.fromGenericDocumentResults(
          genericResult);
    } catch (e) {
      Logger.root.severe(e);
      return GenericDocumentWrappedResults.error(e.toString());
    }
  }

  /// Starts the Ready-to-Use MRZ Scanner UI as full-screen modal UI dialog.
  /// The MRZ Scanner is able to recognize and parse data from the Machine Readable Zone
  /// on ID cards, passports, travel documents, etc.
  static Future<MrzScanningResult> startMrzScanner(
      MrzScannerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();
      const defaultJsonName = "MrzScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var result = await _invoke('startMrzScanner', configuration);
      if (_wasCanceled(result)) {
        return MrzScanningResult.canceled();
      }
      return MrzScanningResult.fromJson(jsonDecode(result));
    } catch (e) {
      Logger.root.severe(e);
      return MrzScanningResult.error(e.toString());
    }
  }

  /// This method takes the default json file and updates the parameters with the given parameters from customer configuration
  /// The approach is needed to always have the full json come to the native platform
  static Future<Map<String, dynamic>> _applyParametersOnDefaultJson(
      String defaultJsonName, Map<String, dynamic> updatedParameters) async {
    var defaultConfiguration = jsonDecode(await rootBundle.loadString(
            'packages/scanbot_sdk/assets/jsonExamples/$defaultJsonName.json'))
        as Map<String, dynamic>;
    updateParameter(updatedParameters, defaultConfiguration);
    return defaultConfiguration;
  }

  static void updateParameter(Map<String, dynamic> updatedParameters,
      Map<String, dynamic> defaultConfiguration) {
    updatedParameters.forEach((key, value) {
      if (value is Map) {
        var map = value as Map<String, dynamic>;
        updateParameter(map, defaultConfiguration[key]);
      } else {
        defaultConfiguration[key] = value;
      }
    });
  }

  /// Starts the Ready-to-Use EHIC Scanner UI as full-screen modal UI dialog.
  /// The EHIC Scanner is able to recognize and parse data from the backside of
  /// European Health Insurance Cards.
  static Future<HealthInsuranceCardRecognitionResult> startEhicScanner(
      HealthInsuranceScannerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();
      const defaultJsonName = "EhicScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var result = await _invoke('startEhicScanner', configuration);
      if (_wasCanceled(result)) {
        return HealthInsuranceCardRecognitionResult.canceled();
      }
      return HealthInsuranceCardRecognitionResult.fromJson(jsonDecode(result));
    } catch (e) {
      Logger.root.severe(e);
      return HealthInsuranceCardRecognitionResult(
          <HealthInsuranceCardField>[], null);
    }
  }

  /// Starts the Ready-to-Use Cropping UI as full-screen modal UI dialog.
  /// The Cropping UI provides the functionality for manual cropping and rotation of the given [page].
  static Future<CroppingResult> startCroppingScreen(
      Page page, CroppingScreenConfiguration config) async {
    try {
      var arguments = config.toJson();
      arguments['page'] = page.toJson();
      var result = await _invoke('startCroppingScreen', arguments);
      if (_wasCanceled(result)) {
        return CroppingResult.canceled();
      }
      return CroppingResult.fromJson(jsonDecode(result));
    } catch (e) {
      Logger.root.severe(e);
      return CroppingResult.error();
    }
  }

  /// Starts the Ready-to-Use License Plate Scanner UI as full-screen modal UI dialog.
  /// The License Plate Scanner is able to recognize european car license plates.
  static Future<LicensePlateScanResult> startLicensePlateScanner(
      LicensePlateScannerConfiguration config) async {
    try {
      var updatedParameters = config.toJson();
      const defaultJsonName = "LicensePlateScreen";
      var configuration = await _applyParametersOnDefaultJson(
          defaultJsonName, updatedParameters);
      var result = await _invoke('startLicensePlateScanner', configuration);
      if (_wasCanceled(result)) {
        return LicensePlateScanResult.canceled();
      }
      return LicensePlateScanResult.fromJson(jsonDecode(result));
    } catch (e) {
      Logger.root.severe(e);
      return LicensePlateScanResult.error(e.toString());
    }
  }
}

typedef Handler = Future Function(MethodCall call);
